#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var child_process = require("child_process");
var glob = require("glob");
var path = require("path");
var yaml = require("js-yaml");
var program = require("commander");
var src;
var repoRelativePath;
var dest = 'docs-ref-autogen';
var packageWhiteList = [];
var packageBlackList = [];
var config;
var pjson = require('../package.json');
var envPath = process.env['APPDATA'] + "/npm/node_modules/ts-reference-ci-scripts";
program
    .version("v" + pjson.version)
    .description('A tool to go through source code and convert it to yaml data model for docfx.')
    .option('--config [value]', 'The config file which will defined more specefied behaviour for the tool')
    .option('--destPath [value]', 'Path to output the autogen files [to docs-ref-autogen].')
    .option('--destPathWithoutSuffix [value]', 'Path to output the autogen files.')
    .arguments('<sourceFolder> [repoConfigPath]')
    .action(function (sourceFolder, repoConfig) {
    src = sourceFolder;
    repoRelativePath = repoConfig;
})
    .parse(process.argv);
if (!src) {
    console.log('Error: The source file folder is not specified!');
    program.help();
}
if (program.config) {
    try {
        config = JSON.parse(program.config);
    }
    catch (e) {
        console.log("json content: " + program.config);
        throw 'Invalid json format of config, stopped!';
    }
}
if (config) {
    if (config.package_white_list) {
        packageWhiteList = config.package_white_list;
    }
    if (config.package_black_list) {
        packageBlackList = config.package_black_list;
    }
    if (config.npm_package_sources && config.npm_package_sources.length) {
        config.npm_package_sources.forEach(function (npmPackageSource) {
            var npmPackageSourceName = npmPackageSource.name.trim();
            if (!npmPackageSource.registry) {
                child_process.execSync("npm install " + npmPackageSourceName + " 2>&1");
            }
            else {
                child_process.execSync("npm install " + npmPackageSourceName + "  --registry " + npmPackageSource.registry + " 2>&1");
            }
            if (fs.existsSync("node_modules/" + npmPackageSourceName)) {
                fs.copySync("node_modules/" + npmPackageSourceName, src + "/" + npmPackageSourceName);
            }
            else {
                var indexOfAtSymbol = npmPackageSourceName.lastIndexOf('@');
                if (indexOfAtSymbol === -1) {
                    throw "Failed to find package source folder for " + npmPackageSourceName;
                }
                var npmPackageSourceFolder = npmPackageSourceName.substring(0, indexOfAtSymbol);
                fs.copySync("node_modules/" + npmPackageSourceFolder, src + "/" + npmPackageSourceFolder);
            }
        });
    }
}
if (program.destPath) {
    if (config && config.target_repo.folder) {
        dest = program.destPath + "/" + config.target_repo.folder + "/" + dest;
    }
    else {
        dest = program.destPath + "/" + dest;
    }
}
if (program.destPathWithoutSuffix) {
    dest = program.destPathWithoutSuffix;
}
// 1. prepare
fs.removeSync(dest);
fs.mkdirpSync(dest);
var packagePaths = glob.sync(path.join(src, '**/package.json'));
var toc = [];
packagePaths.forEach(function (packagePath) {
    if (!fs.existsSync(packagePath) || packagePath.indexOf('node_modules/') >= 0) {
        return;
    }
    var packageName = fs.readJsonSync(packagePath).name;
    var dirOfTypeFile = fs.readJsonSync(packagePath).types;
    if (packageWhiteList.length === 0 || packageWhiteList.indexOf(packageName) >= 0) {
        if (packageBlackList.indexOf(packageName) >= 0) {
            return;
        }
        generatePackageDoc(packagePath, packageName, dest);
    }
});
toc = JSON.parse(JSON.stringify(toc));
fs.writeFileSync(dest + "/toc.yml", yaml.safeDump(toc));
process.exit(0);
function generatePackageDoc(packagePath, packageName, dest) {
    var dir = path.dirname(packagePath);
    console.log("current package: " + packageName);
    var sourceCodeBasePath = "" + dir;
    if (fs.existsSync(dir + "/src")) {
        sourceCodeBasePath = dir + "/src";
    }
    else if (fs.existsSync(dir + "/lib")) {
        sourceCodeBasePath = dir + "/lib";
    }
    else if (fs.readJsonSync(packagePath).types) {
        var typingPath = fs.readJsonSync(packagePath).types;
        if (typingPath.toLowerCase().endsWith('.d.ts')) {
            typingPath = path.dirname(typingPath);
        }
        sourceCodeBasePath = path.join(dir, typingPath);
        if (!fs.existsSync(sourceCodeBasePath)) {
            throw 'incorrect types specification in package.json:' + fs.readJsonSync(packagePath).types;
        }
    }
    else {
        var items = fs.readdirSync(sourceCodeBasePath);
        var numsOfSubDirs = 0;
        var curSubDirs = sourceCodeBasePath;
        for (var i = 0; i < items.length; i++) {
            if (items[i] === 'node_modules') {
                continue;
            }
            var file = sourceCodeBasePath + '/' + items[i];
            var stats = fs.statSync(file);
            if (stats.isDirectory()) {
                numsOfSubDirs++;
                curSubDirs = file;
            }
        }
        if (numsOfSubDirs === 1) {
            sourceCodeBasePath = curSubDirs;
        }
        else {
            throw "No source file for " + packageName;
        }
    }
    var folder = dir.split('/')[1];
    var processed = false;
    var folderIndex = parseInt(folder, 10);
    if (config && config.source_repos && config.source_repos.length && folderIndex !== NaN
        && config.source_repos[folderIndex] && config.source_repos[folderIndex].folder) {
        sourceCodeBasePath = dir + "/" + config.source_repos[folderIndex].folder;
    }
    if (!fs.existsSync(sourceCodeBasePath)) {
        throw "No source file for " + packageName;
    }
    var packageDest = dest + "/" + packageName;
    console.log(packageDest);
    fs.mkdirpSync(packageDest);
    child_process.execSync('npm install 2>&1', { cwd: dir });
    console.log('execute typedoc...');
    child_process.execSync("typedoc --json " + dir + "/api.json " + sourceCodeBasePath + " --module commonjs --includeDeclarations --ignoreCompilerErrors --excludeExternals");
    var basePath = sourceCodeBasePath.replace(src + "/", '');
    var extraCommand = getExtraCommand(config);
    console.log('execute type2docfx...');
    if (config && config.source_repos && config.source_repos.length) {
        var processed_1 = false;
        if (folderIndex !== NaN && config.source_repos[folderIndex]) {
            basePath = basePath.replace(folder + "/", '');
            child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repos[folderIndex].url + " --sourceBranch " + config.source_repos[folderIndex].branch + " --basePath " + basePath + " " + extraCommand);
            processed_1 = true;
        }
        else if (folder) {
            for (var index in config.source_repos) {
                if (folder === config.source_repos[index].folder) {
                    basePath = basePath.replace(folder + "/", '');
                    child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repos[index].url + " --sourceBranch " + config.source_repos[index].branch + " --basePath " + basePath + " " + extraCommand);
                    processed_1 = true;
                    break;
                }
            }
        }
        if (!processed_1) {
            child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " --sourceUrl " + config.source_repos[0].url + " --sourceBranch " + config.source_repos[0].branch + " --basePath " + basePath + " " + extraCommand);
        }
    }
    else {
        child_process.execSync("type2docfx " + dir + "/api.json " + packageDest + " " + extraCommand);
    }
    if (fs.existsSync(packageDest + "/toc.yml")) {
        var subToc = yaml.safeLoad(fs.readFileSync(packageDest + "/toc.yml").toString());
        toc.push(subToc[0]);
        fs.removeSync(packageDest + "/toc.yml");
    }
}
function getExtraCommand(config) {
    var extraCommand = '';
    if (config) {
        if (config.has_module) {
            extraCommand += ' --hasModule';
        }
        if (config.disable_alphabet_order) {
            extraCommand += ' --disableAlphabetOrder';
        }
        if (config.enable_function_level_toc) {
            extraCommand += ' --enableFunctionLevelToc';
        }
    }
    return extraCommand;
}
