#!/usr/bin/env node

import * as fs from 'fs-extra';
import * as child_process from 'child_process';
import * as glob from 'glob';
import * as path from 'path';
import * as yaml from 'js-yaml';
import * as program from 'commander';
import { Config } from './interfaces/Config';

let src: string;
let repoRelativePath: string;
let dest = 'docs-ref-autogen';
let packageWhiteList: string[] = [];
let packageBlackList: string[] = [];
let config: Config;

let pjson = require('../package.json');
let envPath = `${process.env['APPDATA']}/npm/node_modules/ts-reference-ci-scripts`;

program
    .version(`v${pjson.version}`)
    .description('A tool to go through source code and convert it to yaml data model for docfx.')
    .option('--config [value]', 'The config file which will defined more specefied behaviour for the tool')
    .option('--destPath [value]', 'Path to output the autogen files [to docs-ref-autogen].')
    .option('--destPathWithoutSuffix [value]', 'Path to output the autogen files.')
    .arguments('<sourceFolder> [repoConfigPath]')
    .action(function (sourceFolder: string, repoConfig: string) {
        src = sourceFolder;
        repoRelativePath = repoConfig;
    })
    .parse(process.argv);

if (!src) {
    console.log('Error: The source file folder is not specified!');
    program.help();
}

if (program.config) {
    try {
        config = JSON.parse(program.config);
    } catch (e) {
        console.log(`json content: ${program.config}`);
        throw 'Invalid json format of config, stopped!';
    }
}

if (config) {
    if (config.package_white_list) {
        packageWhiteList = config.package_white_list;
    }

    if (config.package_black_list) {
        packageBlackList = config.package_black_list;
    }

    if (config.npm_package_sources && config.npm_package_sources.length) {
        config.npm_package_sources.forEach(npmPackageSource => {
            let npmPackageSourceName = npmPackageSource.name.trim();
            if (!npmPackageSource.registry) {
                child_process.execSync(`npm install ${npmPackageSourceName} 2>&1`);
            } else {
                child_process.execSync(`npm install ${npmPackageSourceName}  --registry ${npmPackageSource.registry} 2>&1`);
            }

            if (fs.existsSync(`node_modules/${npmPackageSourceName}`)) {
                fs.copySync(`node_modules/${npmPackageSourceName}`, `${src}/${npmPackageSourceName}`);
            } else {
                let indexOfAtSymbol = npmPackageSourceName.lastIndexOf('@');
                if (indexOfAtSymbol === -1) {
                    throw `Failed to find package source folder for ${npmPackageSourceName}`;
                }
                let npmPackageSourceFolder = npmPackageSourceName.substring(0, indexOfAtSymbol);
                fs.copySync(`node_modules/${npmPackageSourceFolder}`, `${src}/${npmPackageSourceFolder}`);
            }
        });
    }
}

if (program.destPath) {
    if (config && config.target_repo.folder) {
        dest = `${program.destPath}/${config.target_repo.folder}/${dest}`;
    } else {
        dest = `${program.destPath}/${dest}`;
    }
}

if (program.destPathWithoutSuffix) {
    dest = program.destPathWithoutSuffix;
}

// 1. prepare
fs.removeSync(dest);
fs.mkdirpSync(dest);

let packagePaths = glob.sync(path.join(src, '**/package.json'));
let toc: any[] = [];

packagePaths.forEach(function (packagePath) {
    if (!fs.existsSync(packagePath) || packagePath.indexOf('node_modules/') >= 0) {
        return;
    }
    let packageName = fs.readJsonSync(packagePath).name;
    let dirOfTypeFile = fs.readJsonSync(packagePath).types;
    if (packageWhiteList.length === 0 || packageWhiteList.indexOf(packageName) >= 0) {
        if (packageBlackList.indexOf(packageName) >= 0) {
            return;
        }
        generatePackageDoc(packagePath, packageName, dest);
    }
});

toc = JSON.parse(JSON.stringify(toc));
fs.writeFileSync(`${dest}/toc.yml`, yaml.safeDump(toc));
process.exit(0);

function generatePackageDoc(packagePath: string, packageName: string, dest: string): void {
    let dir = path.dirname(packagePath);
    console.log(`current package: ${packageName}`);
    let sourceCodeBasePath = `${dir}`;
    if (fs.existsSync(`${dir}/src`)) {
        sourceCodeBasePath = `${dir}/src`;
    } else if (fs.existsSync(`${dir}/lib`)) {
        sourceCodeBasePath = `${dir}/lib`;
    } else if (fs.readJsonSync(packagePath).types) {
        let typingPath = fs.readJsonSync(packagePath).types;
        if (typingPath.toLowerCase().endsWith('.d.ts')) {
            typingPath = path.dirname(typingPath);
        }
        sourceCodeBasePath = path.join(dir, typingPath);
        if (!fs.existsSync(sourceCodeBasePath)) {
            throw 'incorrect types specification in package.json:' + fs.readJsonSync(packagePath).types;
        }
    } else {
        let items = fs.readdirSync(sourceCodeBasePath);
        let numsOfSubDirs = 0;
        let curSubDirs = sourceCodeBasePath;
        for (let i = 0; i < items.length; i++) {
            if (items[i] === 'node_modules') {
                continue;
            }
            let file = sourceCodeBasePath + '/' + items[i];
            let stats = fs.statSync(file);
            if (stats.isDirectory()) {
                numsOfSubDirs++;
                curSubDirs = file;
            }
        }
        if (numsOfSubDirs === 1) {
            sourceCodeBasePath = curSubDirs;
        } else {
            throw `No source file for ${packageName}`;
        }
    }
    let folder = dir.split('/')[1];
    let processed = false;
    let folderIndex = parseInt(folder, 10);
    if (config && config.source_repos && config.source_repos.length && folderIndex !== NaN
        && config.source_repos[folderIndex] && config.source_repos[folderIndex].folder) {
        sourceCodeBasePath = `${dir}/${config.source_repos[folderIndex].folder}`;
    }
    if (!fs.existsSync(sourceCodeBasePath)) {
        throw `No source file for ${packageName}`;
    }
    let packageDest = `${dest}/${packageName}`;
    console.log(packageDest);
    fs.mkdirpSync(packageDest);
    child_process.execSync('npm install 2>&1', { cwd: dir });
    console.log('execute typedoc...');
    child_process.execSync(`typedoc --json ${dir}/api.json ${sourceCodeBasePath} --module commonjs --includeDeclarations --ignoreCompilerErrors --excludeExternals`);
    let basePath = sourceCodeBasePath.replace(`${src}/`, '');
    let extraCommand = getExtraCommand(config);
    console.log('execute type2docfx...');
    if (config && config.source_repos && config.source_repos.length) {
        let processed = false;
        if (folderIndex !== NaN && config.source_repos[folderIndex]) {
            basePath = basePath.replace(`${folder}/`, '');
            child_process.execSync(`type2docfx ${dir}/api.json ${packageDest} --sourceUrl ${config.source_repos[folderIndex].url} --sourceBranch ${config.source_repos[folderIndex].branch} --basePath ${basePath} ${extraCommand}`);
            processed = true;
        } else if (folder) {
            for (let index in config.source_repos) {
                if (folder === config.source_repos[index].folder) {
                    basePath = basePath.replace(`${folder}/`, '');
                    child_process.execSync(`type2docfx ${dir}/api.json ${packageDest} --sourceUrl ${config.source_repos[index].url} --sourceBranch ${config.source_repos[index].branch} --basePath ${basePath} ${extraCommand}`);
                    processed = true;
                    break;
                }
            }
        }
        if (!processed) {
            child_process.execSync(`type2docfx ${dir}/api.json ${packageDest} --sourceUrl ${config.source_repos[0].url} --sourceBranch ${config.source_repos[0].branch} --basePath ${basePath} ${extraCommand}`);
        }
    } else {
        child_process.execSync(`type2docfx ${dir}/api.json ${packageDest} ${extraCommand}`);
    }

    if (fs.existsSync(`${packageDest}/toc.yml`)) {
        let subToc = yaml.safeLoad(fs.readFileSync(`${packageDest}/toc.yml`).toString());
        toc.push(subToc[0]);
        fs.removeSync(`${packageDest}/toc.yml`);
    }
}

function getExtraCommand(config: Config): string {
    let extraCommand = '';

    if (config) {
        if (config.has_module) {
            extraCommand += ' --hasModule';
        }

        if (config.disable_alphabet_order) {
            extraCommand += ' --disableAlphabetOrder';
        }

        if (config.enable_function_level_toc) {
            extraCommand += ' --enableFunctionLevelToc';
        }
    }

    return extraCommand;
}
